==> crow-2.11.0.tar.bz2: crow-2.11.0/crow/palette.h <==
/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef CROW_PALETTE_H
#define CROW_PALETTE_H

#include "views.h"

namespace Crow {

class EditorConstructor {

public:
	virtual ~EditorConstructor() { }

	virtual PEditor create() = 0;
	const std::string * getName() { return &name; }
	void setName(const std::string & n) { name= n; }

private:
	std::string name;

};

template <class T> class TEditorConstructor : public EditorConstructor {

public:
	PEditor create() { return NewRefPtr<T>(); }

};

enum PaletteTab {
	TAB_NONE, TAB_WINDOWS, TAB_UI,	
	TAB_STANDARD, TAB_ADDITIONAL, TAB_BINS, 
	TAB_LAYOUTS, TAB_VIEWS, TAB_SELECTORS, 
	TAB_SPECIAL, TAB_DEBUG, TAB_COUNT
};

class Palette {

friend class CrowInit;

public:
	Palette();
	~Palette();

	PaletteEntry & takeEntry(PaletteEntry * entry) { return takePaletteEntry(entry); }
	template <class T> T & takeEntry(T * entry) { takePaletteEntry(entry); return *entry; }

	PaletteEntry * getEntry(TypeId tid) { return getPaletteEntry(tid); }
	template <class T> T * getEntry(TypeId tid) { return dynamic_cast<T *>(getPaletteEntry(tid)); }

	int getEntryCount() { return entries.size(); }

	void addEditor(EditorConstructor * constructor, const std::string & name);
	template <class T> void addEditor(const std::string & name) { addEditor(new TEditorConstructor<T>(), name); }

	EditorId findEditor(const std::string & name);
	PEditor createEditor(EditorId eid);

	TypeId findType(const std::string & name);
	TypeId findType(GType gtype);
	TypeId findType(PGlibObject object);
	std::vector<TypeId> queryTypes(TypeId baseType);
	bool typeIsA(TypeId type, TypeId base);

	bool isScalar(TypeId type);
	bool isEntity(TypeId type);
	bool isObject(TypeId type);
	bool isWidget(TypeId type);

	Glib::ustring createPasteData(TypeId type);

	const std::string * getTabLabel(PaletteTab tab);

	std::string findSignal(const std::string & name, GType type);
	SignalsQuery querySignals(GType type);

	static Palette * get();

	void incrementLeaks(int count) { leaks += count; }
	int getLeaks() { return leaks; }
	void checkLeaks();

	typedef sigc::signal<void> SignalLeaks;

	SignalLeaks & signal_leaks() { return signalLeaks; }

protected:
	PaletteEntry & takePaletteEntry(PaletteEntry * entry);
	PaletteEntry * getPaletteEntry(TypeId tid);
	void init();

	void registerSignals();
	void addingSignals(GType gtype);
	void addSignal(const std::string & name, const std::string & signature);

private:
	std::vector<PaletteEntry *> entries;
	std::vector<EditorConstructor *> editors;
	std::vector<std::string> tabLabels;

	int leaks;
	int oldLeaks;

	SignalLeaks signalLeaks;

	GType sigclass;
	typedef std::multimap<std::string, std::pair<GType, std::string> > Signals;
	Signals signals;

};

class PaletteEntry {

friend class Palette;

public:
	PaletteEntry();
	virtual ~PaletteEntry() { }

	const std::string * getTypeName() { return &typeName; }
	GType getGType() { return gtype; }
	TypeId getType() { return type; }

	EditorId getPropertyEditor() { return propertyEditor; }
	void setPropertyEditor(const std::string & ename);

	int getPriority() { return priority; }
	void setPriority(int pr) { priority= pr; }

	TypeId getIsA() { return isA; }
	void setIsA(const std::string & name);

	std::string suggestName();

	virtual Any createFromGValue(const Glib::ValueBase &);

protected:
	void setTypeName(const std::string & t) { typeName= t; }
	void setGType(GType t) { gtype= t; }
	void setGType(const char * gtname);
	void setType(TypeId t) { type= t; }
	void setPropertyEditor(EditorId editor) { propertyEditor= editor; }

private:
	std::string typeName;
	GType gtype;
	TypeId type;
	EditorId propertyEditor;
	int priority;
	TypeId isA;

};

class ScalarEntry : public PaletteEntry {

public:
	virtual Any createFromString(const Glib::ustring &) = 0;

};

class EnumEntry : public ScalarEntry {

public:
	EnumEntry(const std::string & tname, int size);

	Any createFromGValue(const Glib::ValueBase & val);
	Any createFromString(const Glib::ustring & str);

	typedef std::vector<std::pair<std::string, int> > Values;

	const Values * getValues() { return &values; }
	int getByName(const std::string & name);
	const std::string * getByValue(int value);

	EnumEntry & addValue(const std::string & name, int value);

private:
	Values values;

};

class FlagsEntry : public ScalarEntry {

public:
	FlagsEntry(const std::string & tname, int size);

	Any createFromGValue(const Glib::ValueBase & val);
	Any createFromString(const Glib::ustring & str);

	typedef std::vector<std::pair<std::string, unsigned> > Values;

	const Values * getValues() { return &values; }

	FlagsEntry & addValue(const std::string & name, unsigned value);

	bool valid(unsigned value);

private:
	Values values;

};

typedef unsigned EntryFlags;
const EntryFlags efNone      = 0;
const EntryFlags efTopMaster = 1;
const EntryFlags efAbstract  = 2;
const EntryFlags efInternal  = 4;
const EntryFlags efDebug     = 8;

class EntityEntry : public PaletteEntry {

public:
	EntityEntry();

	PaletteTab getPaletteTab() { return paletteTab; }
	EntityEntry & setPaletteTab(PaletteTab tab) { paletteTab= tab; return *this; }

	EditorId getCanvasEditor() { return canvasEditor; }
	EntityEntry & setCanvasEditor(EditorId eid) { canvasEditor= eid; return *this; }
	EntityEntry & setCanvasEditor(const std::string & ename);

	bool hasFlag(EntryFlags ef) { return (flags & ef) == ef; }
	void setFlag(EntryFlags ef) { flags |= ef; }
	void clearFlag(EntryFlags ef) { flags &= ~ef; }
	void setFlags(EntryFlags ef) { flags= ef; }

	StateFlags getNodeState(const std::string & property);

	virtual PEntityView createView();

protected:
	void prepareView(PEntityView view);

private:
	PaletteTab paletteTab;
	EditorId canvasEditor;
	EntryFlags flags;

};

class ObjectEntry : public EntityEntry {

public:
	Any createFromGValue(const Glib::ValueBase & gvalue);

};

class WidgetEntry : public ObjectEntry {
};

class GlibObjectEntry : public ObjectEntry {

public:
	GlibObjectEntry();

};

class GtkWidgetEntry : public WidgetEntry {

public:
	GtkWidgetEntry();

};

template <class Ttype> class TScalarEntry : public ScalarEntry {

public:
	TScalarEntry(const std::string & tname, GType gt=G_TYPE_INVALID) {
		setTypeName(tname);
		setGType(gt);
		setPropertyEditor(tname);
	}

	Any createFromGValue(const Glib::ValueBase & gvalue) {
		Ttype value= ExtractCppValue<Ttype>(gvalue);
		return CAny::create<Ttype>(getType(), value);
	}

	Any createFromString(const Glib::ustring & str) {
		Ttype value= FromString<Ttype>(str);
		return CAny::create<Ttype>(getType(), value);
	}

};

template <class Tview> class TObjectEntry : public ObjectEntry {

public:
	TObjectEntry(const std::string & tname, GType gt=G_TYPE_INVALID, int pr=0) {
		setTypeName(tname);
		setGType(gt);
		setPropertyEditor("GlibObject");
		setPriority(pr);
	}

	PEntityView createView() {
		PEntityView view= NewRefPtr<Tview>();
		prepareView(view);
		return view;
	}

};

template <class Tview> class TWidgetEntry : public WidgetEntry {

public:
	TWidgetEntry(const std::string & tname, GType gt, PaletteTab ptab=TAB_NONE) {
		setTypeName(tname);
		setGType(gt);
		setPaletteTab(ptab);
		setPropertyEditor("GlibObject");
		setCanvasEditor("canvas|GtkWidget");
	}

	PEntityView createView() {
		PEntityView view= NewRefPtr<Tview>();
		prepareView(view);
		return view;
	}

};

}

#endif

==> crow-2.11.3.tar.bz2: crow-2.11.3/crow/palette.h <==
/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2008 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person 
 *  obtaining a copy of this software and associated documentation 
 *  files (the "Software"), to deal in the Software without 
 *  restriction, including without limitation the rights to use, copy, 
 *  modify, merge, publish, distribute, sublicense, and/or sell copies 
 *  of the Software, and to permit persons to whom the Software is 
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be 
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind, 
 *  express or implied, including but not limited to the warranties of 
 *  merchantability, fitness for a particular purpose and 
 *  noninfringement. In no event shall the authors or copyright 
 *  holders be liable for any claim, damages or other liability, 
 *  whether in an action of contract, tort or otherwise, arising from, 
 *  out of or in connection with the software or the use or other 
 *  dealings in the software.
 */

#ifndef CROW_PALETTE_H
#define CROW_PALETTE_H

#include "views.h"

namespace Crow {

class EditorConstructor {

public:
	virtual ~EditorConstructor() { }

	virtual PEditor create() = 0;
	const std::string * getName() { return &name; }
	void setName(const std::string & n) { name= n; }

private:
	std::string name;

};

template <class T> class TEditorConstructor : public EditorConstructor {

public:
	PEditor create() { return NewRefPtr<T>(); }

};

enum PaletteTab {
	TAB_NONE, TAB_WINDOWS, TAB_UI,	
	TAB_STANDARD, TAB_ADDITIONAL, TAB_BINS, 
	TAB_LAYOUTS, TAB_VIEWS, TAB_SELECTORS, 
	TAB_SPECIAL, TAB_DEBUG, TAB_COUNT
};

class Palette {

friend class CrowInit;

public:
	Palette();
	~Palette();

	PaletteEntry & takeEntry(PaletteEntry * entry) { return takePaletteEntry(entry); }
	template <class T> T & takeEntry(T * entry) { takePaletteEntry(entry); return *entry; }

	PaletteEntry * getEntry(TypeId tid) { return getPaletteEntry(tid); }
	template <class T> T * getEntry(TypeId tid) { return dynamic_cast<T *>(getPaletteEntry(tid)); }

	int getEntryCount() { return entries.size(); }

	void addEditor(EditorConstructor * constructor, const std::string & name);
	template <class T> void addEditor(const std::string & name) { addEditor(new TEditorConstructor<T>(), name); }

	EditorId findEditor(const std::string & name);
	PEditor createEditor(EditorId eid);

	TypeId findType(const std::string & name);
	TypeId findType(GType gtype);
	TypeId findType(PGlibObject object);
	std::vector<TypeId> queryTypes(TypeId baseType);
	bool typeIsA(TypeId type, TypeId base);

	bool isScalar(TypeId type);
	bool isEntity(TypeId type);
	bool isObject(TypeId type);
	bool isWidget(TypeId type);

	Glib::ustring createPasteData(TypeId type);

	const std::string * getTabLabel(PaletteTab tab);

	std::string findSignal(const std::string & name, GType type);
	SignalsQuery querySignals(GType type);

	static Palette * get();

	void incrementLeaks(int count) { leaks += count; }
	int getLeaks() { return leaks; }
	void checkLeaks();

	typedef sigc::signal<void> SignalLeaks;

	SignalLeaks & signal_leaks() { return signalLeaks; }

protected:
	PaletteEntry & takePaletteEntry(PaletteEntry * entry);
	PaletteEntry * getPaletteEntry(TypeId tid);
	void init();

	void registerSignals();
	void addingSignals(GType gtype);
	void addSignal(const std::string & name, const std::string & signature);

private:
	std::vector<PaletteEntry *> entries;
	std::vector<EditorConstructor *> editors;
	std::vector<std::string> tabLabels;

	int leaks;
	int oldLeaks;

	SignalLeaks signalLeaks;

	GType sigclass;
	typedef std::multimap<std::string, std::pair<GType, std::string> > Signals;
	Signals signals;

};

class PaletteEntry {

friend class Palette;

public:
	PaletteEntry();
	virtual ~PaletteEntry() { }

	const std::string * getTypeName() { return &typeName; }
	GType getGType() { return gtype; }
	TypeId getType() { return type; }

	EditorId getPropertyEditor() { return propertyEditor; }
	void setPropertyEditor(const std::string & ename);

	int getPriority() { return priority; }
	void setPriority(int pr) { priority= pr; }

	TypeId getIsA() { return isA; }
	void setIsA(const std::string & name);

	std::string suggestName();

	virtual Any createFromGValue(const Glib::ValueBase &);

protected:
	void setTypeName(const std::string & t) { typeName= t; }
	void setGType(GType t) { gtype= t; }
	void setGType(const char * gtname);
	void setType(TypeId t) { type= t; }
	void setPropertyEditor(EditorId editor) { propertyEditor= editor; }

private:
	std::string typeName;
	GType gtype;
	TypeId type;
	EditorId propertyEditor;
	int priority;
	TypeId isA;

};

class ScalarEntry : public PaletteEntry {

public:
	virtual Any createFromString(const Glib::ustring &) = 0;

};

class EnumEntry : public ScalarEntry {

public:
	EnumEntry(const std::string & tname);

	Any createFromGValue(const Glib::ValueBase & val);
	Any createFromString(const Glib::ustring & str);

	typedef std::list<std::pair<std::string, int> > Values;

	const Values * getValues() { return &values; }
	int getByName(const std::string & name);
	const std::string * getByValue(int value);

	EnumEntry & addValue(const std::string & name, int value);

private:
	Values values;

};

class FlagsEntry : public ScalarEntry {

public:
	FlagsEntry(const std::string & tname);

	Any createFromGValue(const Glib::ValueBase & val);
	Any createFromString(const Glib::ustring & str);

	typedef std::list<std::pair<std::string, Bits> > Values;

	const Values * getValues() { return &values; }

	FlagsEntry & addValue(const std::string & name, Bits value);

	bool valid(Bits value);

private:
	Values values;

};

typedef Bits EntryFlags;
const EntryFlags efNone      = 0;
const EntryFlags efTopMaster = 1;
const EntryFlags efAbstract  = 2;
const EntryFlags efInternal  = 4;
const EntryFlags efDebug     = 8;

class EntityEntry : public PaletteEntry {

public:
	EntityEntry();

	PaletteTab getPaletteTab() { return paletteTab; }
	EntityEntry & setPaletteTab(PaletteTab tab) { paletteTab= tab; return *this; }

	EditorId getCanvasEditor() { return canvasEditor; }
	EntityEntry & setCanvasEditor(EditorId eid) { canvasEditor= eid; return *this; }
	EntityEntry & setCanvasEditor(const std::string & ename);

	bool hasFlag(EntryFlags ef) { return Includes(flags, ef); }
	void setFlag(EntryFlags ef) { Addition(&flags, ef); }
	void clearFlag(EntryFlags ef) { Subtraction(&flags, ef); }
	void setFlags(EntryFlags ef) { flags= ef; }

	StateFlags getNodeState(const std::string & property);

	virtual PEntityView createView();

protected:
	void prepareView(PEntityView view);

private:
	PaletteTab paletteTab;
	EditorId canvasEditor;
	EntryFlags flags;

};

class ObjectEntry : public EntityEntry {

public:
	Any createFromGValue(const Glib::ValueBase & gvalue);

};

class WidgetEntry : public ObjectEntry {
};

class GlibObjectEntry : public ObjectEntry {

public:
	GlibObjectEntry();

};

class GtkWidgetEntry : public WidgetEntry {

public:
	GtkWidgetEntry();

};

template <class Ttype> class TScalarEntry : public ScalarEntry {

public:
	TScalarEntry(const std::string & tname, GType gt=G_TYPE_INVALID) {
		setTypeName(tname);
		setGType(gt);
		setPropertyEditor(tname);
	}

	Any createFromGValue(const Glib::ValueBase & gvalue) {
		Ttype value= ExtractCppValue<Ttype>(gvalue);
		return CAny::create<Ttype>(getType(), value);
	}

	Any createFromString(const Glib::ustring & str) {
		Ttype value= FromString<Ttype>(str);
		return CAny::create<Ttype>(getType(), value);
	}

};

template <class Tview> class TObjectEntry : public ObjectEntry {

public:
	TObjectEntry(const std::string & tname, GType gt=G_TYPE_INVALID, int pr=0) {
		setTypeName(tname);
		setGType(gt);
		setPropertyEditor("GlibObject");
		setPriority(pr);
	}

	PEntityView createView() {
		PEntityView view= NewRefPtr<Tview>();
		prepareView(view);
		return view;
	}

};

template <class Tview> class TWidgetEntry : public WidgetEntry {

public:
	TWidgetEntry(const std::string & tname, GType gt, PaletteTab ptab=TAB_NONE) {
		setTypeName(tname);
		setGType(gt);
		setPaletteTab(ptab);
		setPropertyEditor("GlibObject");
		setCanvasEditor("canvas|GtkWidget");
	}

	PEntityView createView() {
		PEntityView view= NewRefPtr<Tview>();
		prepareView(view);
		return view;
	}

};

}

#endif
